# -*- coding:utf-8 -*-
import re
import random
import string
import time
import datetime
from types import FunctionType
import inspect
import os
import sys
from .log import warn


def isBase64(content: str) -> bool:
    _reg = "^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$"
    group = re.match(_reg, content)
    if group != None:
        return True
    return False


def debug():
    frameList = inspect.stack()
    sss = ["{}.{}\t{}".format(i.filename, i.function, i.lineno) for i in frameList]
    print(len(frameList))
    warn('\n' + '\n'.join(sss))


def getRandomStr(length: int, scope: str = string.ascii_letters+string.digits) -> str:
    return ''.join(random.sample(scope, length))


def generate_id(*_):
    return time.time_ns()


def getDateFolder(format: str = "%Y/%m/%d"):
    """
    获得当前日期目录:
    2023/12/01
    """
    time = datetime.datetime.now()
    return f"{time.strftime(format)}"


def isCallable(func):
    return isinstance(func, FunctionType)
    return callable(func)  # 返回true 也不一定能调用成功/返回失败一定调用失败
    return type(func) is FunctionType
    return hasattr(func, "__call__")


def is_async(func):
    """
    方法是否是异步的
    """
    return inspect.iscoroutinefunction(func) or inspect.isasyncgenfunction(func)


def getWorkDirectory():
    """
    获取工作目录
    """
    current_directory = os.getcwd()
    return current_directory


def find_files(root, filterDirFunction, filterFileFunction, *ignoreDirs: str):
    """
    root: 根目录
    filterDirFunction   返回bool的方法,参数 文件夹名
    filterFileFunction  返回bool的方法,参数 文件名
                       可以使用  lambda f:f.endswith(extension)
    RETURN  root  fdirs, ffile

    查找文件或文件夹
    os.path.join(root, file)

    """
    for root, dirs, files in os.walk(root):
        for ignore in ignoreDirs:
            if ignore in dirs:
                dirs.remove(ignore)
        fdirs = list(filter(filterDirFunction, dirs)
                     ) if filterDirFunction != None else dirs
        ffile = list(filter(filterFileFunction, files)
                     ) if filterFileFunction != None else files
        yield root, fdirs, ffile


def get_parent_dir(path: str, rank: int = 1):
    """
    获取上级目录
    """
    result: str = path
    stop = rank+1
    for _ in range(1, stop):
        result = os.path.dirname(result)
    return result


def getApplcationPath(__file__):
    # print("如果脚本被编译成.pyc文件运行或者使用了一些打包工具（如PyInstaller），那么__file__可能不会返回源.py文件的路径，而是编译后的文件或临时文件的路径")
    # 获取当前文件的完整路径
    if getattr(sys, 'frozen', False):
        # 如果应用程序是冻结的，获取可执行文件的路径
        application_path = os.path.dirname(sys.executable)
    else:
        # 否则，获取原始脚本的路径
        application_path = os.path.dirname(os.path.abspath(__file__))

    return application_path



import asyncio
from threading import Thread
from time import sleep, ctime
from co6co.utils import log
from functools import partial
from types import FunctionType
from co6co.utils import isCallable

class ThreadEvent:
	"""
	线程Event loop 
	Run Event Loop in different thread.
	
	## 因某些原因写了该类 
	## 1. asyncio.run 在 没有正在运行的事件循环 的情况下运行协程的
    ##wx_user_dict:list[dict]=asyncio.run(bll.get_subscribe_alarm_user(config.appid)) 
    ##alarm= alarm_bll(app)
    ##po:bizAlarmTypePO=asyncio.run(alarm.get_alram_type_desc(po.alarmType))
    ## 2. 正在运行的事件循环
    ## This event loop is already running   
    ## import nest_asyncio 
    ##loop = asyncio.get_event_loop()
    ##wx_user_dict:list[dict]=loop.run_until_complete(bll.get_subscribe_alarm_user(config.appid)) 
	
    ## 3. 创建任务
    ##task=asyncio.create_task(bll.get_subscribe_alarm_user(config.appid))
    ##wx_user_dict:list[dict]=asyncio.run(task) 
    ## 4.底层使用
    ##asyncio.ensure_future(coro())
    ## 5. 
	"""
	@property
	def loop(self):
		return self._loop
	
	bck:FunctionType=None

	def __init__(self,threadName:str=None,quitBck:FunctionType=None):
		self._loop =asyncio.new_event_loop() 
		#log.warn(f"ThreadEventLoop:{id(self._loop)}")
		Thread(target=self._start_background, daemon=True,name=threadName) .start()
		self.bck=quitBck

	def _start_background(self):
		asyncio.set_event_loop(self.loop)
		self._loop.run_forever() 
		if self.bck !=None and isCallable(self.bck):self.bck()
		 
	def runTask(self, tastFun , *args, **kwargs):
		#log.warn(f"ThreadEventLoop22:{id(self._loop)}")
		task=asyncio.run_coroutine_threadsafe(tastFun(*args, **kwargs), loop=self._loop)
		return task.result()
	
	def _shutdown(self):
		self._loop.stop()
		#print("running:", self._loop.is_running()) #True
		#print("closed.", self._loop.is_closed())   #False
		  
	def close(self): 
		self._loop.call_soon_threadsafe(partial(self._shutdown )) 
	def __del__(self): 
		self._loop.close()
		#log.warn("closed")
		#print("running:", self._loop.is_running()) #False
		#print("closed.", self._loop.is_closed())   #True
		